import { useLocation, useSearch } from "wouter";
import { Button } from "@/components/ui/button.tsx";
import { GalleryCard } from "@/components/gallery/GalleryCard.tsx";
import TagBadge from "@/components/gallery/TagBadge.tsx";
import { useSearchGalleryInfinite } from "../hook/useSearchGallery.ts";
import { Spinner } from "../components/Spinner.tsx";
import TagInput from "@/components/gallery/TagInput.tsx";
import { useEffect, useRef, useState } from "react";
import { Separator } from "@/components/ui/separator.tsx";
import { useVirtualizer } from "@tanstack/react-virtual";

export default function Gallery() {
    const search = useSearch();
    const searchParams = new URLSearchParams(search);
    const word = searchParams.get("word") ?? undefined;
    const tags = searchParams.get("allow_tag") ?? undefined;
    const limit = searchParams.get("limit");
    const cursor = searchParams.get("cursor");
    const { data, error, isLoading, size, setSize } = useSearchGalleryInfinite({
        word, tags,
        limit: limit ? Number.parseInt(limit) : undefined,
        cursor: cursor ? Number.parseInt(cursor) : undefined
    });
    const parentRef = useRef<HTMLDivElement>(null);
    const virtualizer = useVirtualizer({
        count: size,
        // biome-ignore lint/style/noNonNullAssertion: <explanation>
        getScrollElement: () => parentRef.current!,
        estimateSize: (index) => {
            const docs = data?.[index];
            if (!docs) return 8;
            return docs.data.length * (200 + 8) + 37 + 8;
        },
        overscan: 1,
    });

    const virtualItems = virtualizer.getVirtualItems();
    useEffect(() => {
        const lastItems = virtualItems.slice(-1);
        // console.log(virtualItems);
        if (lastItems.some(x => x.index >= size - 1)) {
            const last = lastItems[0];
            const docs = data?.[last.index];
            if (docs?.hasMore) {
                setSize(size + 1);
            }
        }
    }, [virtualItems, setSize, size, data]);

    if (isLoading) {
        return <div className="p-4">Loading...</div>
    }
    if (error) {
        return <div className="p-4">Error: {String(error)}</div>
    }
    if (!data) {
        return <div className="p-4">No data</div>
    }


    const isLoadingMore = data && size > 0 && (data[size - 1] === undefined);
    const isReachingEnd = data && data[size - 1]?.hasMore === false;

    return (<div className="p-4 grid gap-2 overflow-auto h-dvh items-start content-start" ref={parentRef}>
        <Search />
        {(word || tags) &&
            <div className="bg-primary rounded-full p-1 sticky top-0 shadow-md z-20">
                {word && <span className="text-primary-foreground ml-4">Search: {word}</span>}
                {tags && <span className="text-primary-foreground ml-4">Tags: <ul className="inline-flex">{
                    tags.split(",").map(x => <TagBadge tagname={x} key={x} />)}
                </ul></span>}
            </div>
        }
        {data?.length === 0 && <div className="p-4 text-3xl">No results</div>}
        <div className="w-full relative"
            style={{ height: virtualizer.getTotalSize() }}>
            {// TODO: date based grouping
                virtualItems.map((item) => {
                    const isLoaderRow = item.index === size - 1 && isLoadingMore;
                    if (isLoaderRow) {
                        return <div key={item.index} className="w-full flex justify-center top-0 left-0 absolute"
                            style={{
                                height: `${item.size}px`,
                                transform: `translateY(${item.start}px)`
                            }}>
                            <Spinner />
                        </div>;
                    }
                    const docs = data[item.index];
                    if (!docs) return null;
                    return <div className="w-full grid gap-2 content-start top-0 left-0 absolute mb-2" key={item.index}
                        style={{
                            height: `${item.size}px`,
                            transform: `translateY(${item.start}px)`
                        }}>
                        {docs.startCursor && <div>
                            <h3 className="text-3xl">Start with {docs.startCursor}</h3>
                            <Separator />
                        </div>}
                        {docs?.data?.map((x) => {
                            return (
                                <GalleryCard doc={x} key={x.id} />
                            );
                        })}
                    </div>
                })
            }
        </div>
    </div>
    );
}

function Search() {
    const search = useSearch();
    const [, navigate] = useLocation();
    const searchParams = new URLSearchParams(search);
    const [tags, setTags] = useState(searchParams.get("allow_tag")?.split(",") ?? []);
    const [word, setWord] = useState(searchParams.get("word") ?? "");
    return <div className="flex space-x-2">
        <TagInput className="flex-1" input={word} onInputChange={setWord}
            tags={tags} onTagsChange={setTags}
        />
        <Button className="flex-none" onClick={() => {
            const params = new URLSearchParams();
            if (tags.length > 0) {
                for (const tag of tags) {
                    params.append("allow_tag", tag);
                }
            }
            if (word) {
                params.set("word", word);
            }
            navigate(`/search?${params.toString()}`);
        }}>Search</Button>
    </div>;
}

